#  compute the eLC MLE for discrete Gaussian example in manuscript

#--------------------------
#  load required functions (assuming files in working directory)
#--------------------------

dyn.load("convhullnew.dll")  
source("functions.R")


#--------------------------
# generate discrete Gaussian sample, result written to emp.txt
#--------------------------
set.seed(20-11-1951)
xx 		<- 	rgaussiandscr(n=500)  # create discrete gaussian random sample
# optional input parameters:
# n: sample size
# N: (-N,N)^2 is the range for calculating the true PMF, by default N=100
# mu: mean of bivariate discrete gaussian, by default mu= c(1,2)      
# sig1: standard deviation of bivariate discrete gaussian, by default sig1 = 2
# sig2: standard deviation of bivariate discrete gaussian, by default sig2 = 5
# rho: correlation coefficient of bivariate discrete gaussian, by default rho = 0.6
xd		<-	as.matrix(aggregate(list(emp=rep(1,nrow(data.frame(xx)))), data.frame(xx), length))
write.table(xd, "emp.txt", sep = ",", row.names = F, col.names = T)   


#--------------------------
# estimate eLC MLE, result written to mle.txt
#--------------------------

xd 	<- as.matrix(read.table("emp.txt", header = TRUE, sep=","))
mle <- eLC(xd)   # compute eLC MLE  
# xd: input data, first two columns give the unique observed location, last column is the empirical count
colnames(mle) <- c("X","Y","elc")
write.table(mle, "mle.txt",sep = ",", row.names = F,col.names = T)

#  plot results for the computed the eLC MLE for discrete Gaussian example in manuscript

#--------------------------
#  load required functions (assuming files in working directory)
#--------------------------
#source("rgauss.R")

#--------------------------
# plot of eLC MLE
#--------------------------
mle 	<- read.table("mle.txt", sep=",", header = TRUE)
par(las = 1, oma = c(0, 0, 0, 0), cex = 0.9, mar = c(2, 2, 0, 0))
plot(0, 0, type = "n", xlab = "", ylab = "",   main = "",
     xlim = c(min(mle[,1])-1,max(mle[,1])+1),  
     ylim = c(min(mle[,2])-1,max(mle[,2])+1))
for(i in 1:length(mle[,1])){
  rect(mle[i,1]-0.5,mle[i,2]-0.5, mle[i,1]+0.5, mle[i,2]+0.5, col=gray((max(mle[,3])-mle[i,3])/max(mle[,3])))
}


#--------------------------
# plot of empirical PMF
#--------------------------
xd 	<- read.table("emp.txt", sep=",", header = TRUE)
par(las = 1, oma = c(0, 0, 0, 0), cex = 0.9, mar = c(2, 2, 0, 0))
plot(0, 0, type = "n", xlab = "", ylab = "",   main = "",
     xlim = c(min(xd[,1])-1,max(xd[,1])+1),  
     ylim = c(min(xd[,2])-1,max(xd[,2])+1))
for(i in 1:length(xd[,1])){
  rect(xd[i,1]-0.5,xd[i,2]-0.5, xd[i,1]+0.5, xd[i,2]+0.5, col=gray((max(xd[,3])-xd[i,3])/max(xd[,3])))
}

#--------------------------
# plot of marginals 
#--------------------------
dtrue <- pNgaussiandscr(N=100)  # get true pmf
# input parameter description:
# N: (-N,N)^2 is the range for calculating the true PMF, by default N=100
# mu: mean of bivariate discrete gaussian, by default mu= c(1,2)      
# sig1: standard deviation of bivariate discrete gaussian, by default sig1 = 2
# sig2: standard deviation of bivariate discrete gaussian, by default sig2 = 5
# rho: correlation coefficient of bivariate discrete gaussian, by default rho = 0.6

d1 <- merge(mle, xd, by = c("X","Y"), all = TRUE)
d <- merge(d1,dtrue, by = c("X","Y"), all = TRUE)
d[is.na(d[,"true"]) == TRUE,"true"] <- 0
d[is.na(d[,"emp"]) == TRUE,"emp"] <- 0
d[is.na(d[,"elc"]) == TRUE,"elc"] <- 0
d$emp <- d$emp/sum(d$emp)

# marginal for x
supp1 <- sort(unique(d$X))
m1 = matrix(0,nrow = length(supp1),ncol=4)
m1[,1] <- supp1
for (i in 1:length(supp1)) {
  m1[i,2] <- sum(d[d$X == supp1[i],]$emp)
  m1[i,3] <- sum(d[d$X == supp1[i],]$true)
  m1[i,4] <- sum(d[d$X == supp1[i],]$elc)
}
colnames(m1) <- c("X", "emp","true","elc")
head(m1)

par(las = 1, oma = c(0, 0, 0, 0), cex = 0.9, mar = c(4.5, 3, 1, 1))
plot(0, 0, type = "n", xlim = c(-6, 7), xlab = "", xaxt = "n", ylim = c(0, max(m1[,2:4]) + 0.03), ylab = "", main = "")
axis(1, at = min(supp1):max(supp1), labels = min(supp1):max(supp1))
off <- 0.4
rect(xleft = min(supp1):max(supp1) - off, ybottom = 0, xright = min(supp1):max(supp1) + off, ytop = m1[,"emp"], col = grey(0.8), border = grey(0.8)) 
lines(min(supp1):max(supp1), m1[,"elc"], col = "black", type = "b", lwd = 1)   
lines(min(supp1):max(supp1), m1[,"true"], col = "black", type = "l", lwd = 2, lty = 1)   
legend("topright", c("empirical", "eLC MLE", "truth"), pch = c(15, 1, -1), lty = c(-1, 1, 1), col = c(grey(0.8), "black", "black"), bty = "n", lwd = c(-1, 1, 2))


# marginal for y
supp2 <- sort(unique(d$Y))
m2 = matrix(0,nrow = length(supp2),ncol=4)
m2[,1] <- supp2
for (i in 1:length(supp2)) {
  m2[i,2] <- sum(d[d$Y == supp2[i],]$emp)
  m2[i,3] <- sum(d[d$Y == supp2[i],]$true)
  m2[i,4] <- sum(d[d$Y == supp2[i],]$elc)
}
colnames(m2) <- c("Y", "emp","true","elc")
head(m2)

par(las = 1, oma = c(0, 0, 0, 0), cex = 0.9, mar = c(4.5, 3, 1, 1))
plot(0, 0, type = "n", xlim = c(-13, 18), xlab = "", xaxt = "n", ylim = c(0, max(m2[,2:4]) + 0.03), ylab = "", main = "")
axis(1, at = min(supp2):max(supp2), labels = min(supp2):max(supp2))
off <- 0.4
rect(xleft = min(supp2):max(supp2) - off, ybottom = 0, xright = min(supp2):max(supp2) + off, ytop = m2[,"emp"], col = grey(0.8), border = grey(0.8)) 
lines(min(supp2):max(supp2), m2[,"elc"], col = "black", type = "b", lwd = 1)   
lines(min(supp2):max(supp2), m2[,"true"], col = "black", type = "l", lwd = 2, lty = 1)   
legend("topright", c("empirical", "eLC MLE", "truth"), pch = c(15, 1, -1), lty = c(-1, 1, 1), col = c(grey(0.8), "black", "black"), bty = "n", lwd = c(-1, 1, 2))

